#include "lowlevel.h"
#include "highlevel.h"

void LCD_cursor ( unsigned int address )
{
	LCD_WriteCmd(0x46);									// CSRW, Sets cursor address.
	LCD_Write(address&0xFF);							// LO byte
	LCD_Write((address>>8)&0xFF);						// HI byte
}

void LCD_fill(unsigned int address,unsigned char width,unsigned char height,unsigned char pattern)
{
unsigned int w;

   while(height--){
     LCD_cursor(address);
     LCD_WriteCmd(0x42);									// MWRITE
     w=width;
     while(w--)
       LCD_Write(pattern);							// fill with pattern
     address+=40;
   }
}


void LCD_printat (unsigned char row, unsigned int col, char *ptr)
{
	LCD_cursor ((row<<5)+(row<<3)+col);	// 40*y+col						// set cursor address
	LCD_WriteCmd(0x42);									// MWRITE
 	while (*ptr)
		LCD_Write (*ptr++);
}

/* Plot image data, 8 pixels per byte, MSB left */

void LCD_plot (unsigned int x, unsigned char y)
{
unsigned char strip,bt,dat;
unsigned int addr;

	strip=(unsigned char)(x>>3);											// strip = x/8
	bt=(unsigned char)x & 0x07;									// bit = remainder
	addr=(y<<5)+(y<<3);                     // 40*y
	addr+=1200+strip;
	LCD_cursor(addr);										// address strip in gfx screen
	LCD_WriteCmd(0x43);								// MREAD
	dat=LCD_ReadData();								// get data at that address
	dat|=(0x80>>bt);								// set requested bit (MSB left)
	LCD_cursor(addr);										// address strip in gfx screen again
	LCD_WriteCmd(0x42);								// MWRITE
	LCD_Write(dat);
}


void LCD_icon(unsigned int address,unsigned char width,unsigned char height,__code unsigned char * imgdata)
{
unsigned int w;

   while(height--){
     LCD_cursor(address);
     LCD_WriteCmd(0x42);									// MWRITE
     w=width;
     while(w--)
	  LCD_Write(*(imgdata++));
     address+=40;
   }
}


// Bresenham Line Algorithm

void LCD_line(unsigned int x1,unsigned char y1,unsigned int x2,unsigned char y2)
{
unsigned char y;
int dy,incx,incy;
unsigned int x;
int dx,balance;

        if (x2 >= x1)
        {
                dx = x2 - x1;
                incx = 1;
        }
        else
        {
                dx = x1 - x2;
                incx = -1;
        }

        if (y2 >= y1)
        {
                dy = y2 - y1;
                incy = 1;
        }
        else
        {
                dy = y1 - y2;
                incy = -1;
        }

        x = x1;
        y = y1;

        if (dx >= dy)
        {
                dy <<= 1;
                balance = dy - dx;
                dx <<= 1;

                while (x != x2)
                {
                        LCD_plot(x, y);
                        if (balance >= 0)
                        {
                                y += incy;
                                balance -= dx;
                        }
                        balance += dy;
                        x += incx;
                } LCD_plot(x, y);
        }
        else
        {
                dx <<= 1;
                balance = dx - dy;
                dy <<= 1;

                while (y != y2)
                {
                        LCD_plot(x, y);
                        if (balance >= 0)
                        {
                                x += incx;
                                balance -= dy;
                        }
                        balance += dx;
                        y += incy;
                } LCD_plot(x, y);
        }
}

void LCD_rectangle(unsigned int x1,unsigned char y1,unsigned int x2,unsigned char y2)
{
	LCD_line(x1,y1,x2,y1);
	LCD_line(x2,y1,x2,y2);
	LCD_line(x2,y2,x1,y2);
	LCD_line(x1,y2,x1,y1);
}

